package com.xlht.xlhtproject.service.impl;


import com.xlht.xlhtproject.domain.SmartAdd;
import com.xlht.xlhtproject.domain.TTbDu;
import com.xlht.xlhtproject.domain.TXinyiMedicineParam;
import com.xlht.xlhtproject.mapper.TShuju1Mapper;
import com.xlht.xlhtproject.mapper.TTbDuMapper;
import com.xlht.xlhtproject.mapper.TXinyiMedicineParamMapper;
import com.xlht.xlhtproject.service.ITXinyiMedicineParamService;
import com.xlht.xlhtproject.utils.CalculateUtils;
import com.xlht.xlhtproject.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.annotations.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;

import static com.xlht.xlhtproject.enums.MyConstants.LONG_1;


/**
 * 智能投药参数设置Service业务层处理
 * 
 * @author slibra
 * @date 2024-09-09
 */
@Slf4j
@Service
public class TXinyiMedicineParamServiceImpl implements ITXinyiMedicineParamService
{
    @Autowired
    private TXinyiMedicineParamMapper tXinyiMedicineParamMapper;

    @Autowired
    private TShuju1Mapper shuju1Mapper;

    @Autowired
    private CalculateUtils calculateUtils;

    @Autowired
    private TTbDuMapper tTbDuMapper;


    /**
     * 查询智能投药参数设置
     * 
     * @param id 智能投药参数设置主键
     * @return 智能投药参数设置
     */
    @Override
    public TXinyiMedicineParam selectTXinyiMedicineParamById(Long id)
    {
        return tXinyiMedicineParamMapper.selectTXinyiMedicineParamById(id);
    }

    /**
     * 查询智能投药参数设置列表
     * 
     * @param tXinyiMedicineParam 智能投药参数设置
     * @return 智能投药参数设置
     */
    @Override
    public List<TXinyiMedicineParam> selectTXinyiMedicineParamList(TXinyiMedicineParam tXinyiMedicineParam)
    {
        return tXinyiMedicineParamMapper.selectTXinyiMedicineParamList(tXinyiMedicineParam);
    }

    /**
     * 新增智能投药参数设置
     * 
     * @param tXinyiMedicineParam 智能投药参数设置
     * @return 结果
     */
    @Override
    public int insertTXinyiMedicineParam(TXinyiMedicineParam tXinyiMedicineParam)
    {
        tXinyiMedicineParam.setCreateTime(DateUtils.getNowDate());
        return tXinyiMedicineParamMapper.insertTXinyiMedicineParam(tXinyiMedicineParam);
    }

    /**
     * 修改智能投药参数设置
     * 
     * @param tXinyiMedicineParam 智能投药参数设置
     * @return 结果
     */
    @Override
    public int updateTXinyiMedicineParam(TXinyiMedicineParam tXinyiMedicineParam)
    {
        tXinyiMedicineParam.setUpdateTime(DateUtils.getNowDate());
        return tXinyiMedicineParamMapper.updateTXinyiMedicineParam(tXinyiMedicineParam);
    }

    /**
     * 批量删除智能投药参数设置
     * 
     * @param ids 需要删除的智能投药参数设置主键
     * @return 结果
     */
    @Override
    public int deleteTXinyiMedicineParamByIds(Long[] ids)
    {
        return tXinyiMedicineParamMapper.deleteTXinyiMedicineParamByIds(ids);
    }

    /**
     * 删除智能投药参数设置信息
     * 
     * @param id 智能投药参数设置主键
     * @return 结果
     */
    @Override
    public int deleteTXinyiMedicineParamById(Long id)
    {
        return tXinyiMedicineParamMapper.deleteTXinyiMedicineParamById(id);
    }

    @Override
    public TXinyiMedicineParam getLatestRecord() {
        TXinyiMedicineParam latestRecord = this.tXinyiMedicineParamMapper.getLatestRecord();
        //todo 额外查询 碳源投加瞬时流量
        if(!Objects.isNull(latestRecord)){
            this.addExtra(latestRecord);
        }
        return latestRecord;
    }

    private void addExtra(TXinyiMedicineParam latestRecord) {
        //2024年09月20日13:42:18  有可能用户选择的是仪表或者化验，需要获取实时数据
        //获取最新的实时数据
        SmartAdd smartAdd = this.shuju1Mapper.getNewestData();
        if(!Objects.isNull(smartAdd)){
            //获取对应的类型
            Integer hycXsyType = latestRecord.getHycXsyType();
            Integer qycAdType = latestRecord.getQycAdType();
            Integer qycYxyType = latestRecord.getQycYxyType();
            Integer jsCodType = latestRecord.getJsCodType();
            Integer jsLlType = latestRecord.getJsLlType();
            Integer jsTnType = latestRecord.getJsTnType();
            if(!Objects.isNull(hycXsyType)){
                if(1 == hycXsyType){//化验
                    latestRecord.setHycXsyOne(smartAdd.getHyXsyHYOne());
                    latestRecord.setHycXsyTwo(smartAdd.getHyXsyHYTwo());
                }else if(2 == hycXsyType){//预测
                    latestRecord.setHycXsyOne(smartAdd.getHyXsyYCOne());
                    latestRecord.setHycXsyTwo(smartAdd.getHyXsyYCTwo());
                }
            }
            if(!Objects.isNull(qycAdType) && 1 == qycAdType){//化验
                latestRecord.setQycAdOne(smartAdd.getQyAdHYOne());
                latestRecord.setQycAdTwo(smartAdd.getQyAdHYTwo());
            }
            if(!Objects.isNull(qycYxyType) && 1 == qycYxyType){//化验
                latestRecord.setQycYxyOne(smartAdd.getQyXsyHYOne());
                latestRecord.setQycYxyTwo(smartAdd.getQyXsyHYTwo());
            }
            if(!Objects.isNull(jsCodType)){
                if(1 == jsCodType){//仪表
                    latestRecord.setJsCodOne(smartAdd.getJsCodYB());
                    latestRecord.setJsCodTwo(smartAdd.getJsCodYB());
                }else if(2 == jsCodType){//化验
                    latestRecord.setJsCodOne(smartAdd.getJsCodHY());
                    latestRecord.setJsCodTwo(smartAdd.getJsCodHY());
                }
            }
            if(!Objects.isNull(jsLlType) && 1 == jsLlType){//仪表
                latestRecord.setJsLlOne(smartAdd.getJslYB());
                latestRecord.setJsLlTwo(smartAdd.getJslYB());
            }
            if(!Objects.isNull(jsTnType) && 1 == jsTnType){//仪表
                latestRecord.setJsTnOne(smartAdd.getJsTnYB());
                latestRecord.setJsTnTwo(smartAdd.getJsTnYB());
            }
            //计算实时的碳源投加量 (2个池子需要分开存 ToDo)
            BigDecimal val = this.calculateUtils.calculateMedicineByLastRecord(latestRecord, 1);
            if(!Objects.isNull(val) && val.compareTo(BigDecimal.ZERO) < 0){
                log.info("计算得到的碳源投加量是负数，按0处理");
                val = BigDecimal.ZERO;
            }
            latestRecord.setMedicineAmount(val);
            //获取2个池子的瞬时投药流量
            TTbDu tTbDu = tTbDuMapper.selectTTbDuByID(LONG_1);
            if(!Objects.isNull(tTbDu)){
                latestRecord.setTytjTransientLLOne(tTbDu.getC1TjLl());
                latestRecord.setTytjTransientLLTwo(tTbDu.getC2TjLl());
            }
        }
    }
}
